'''
 ............................................................. 

 Filter to create a NEW image from the INPUT DRAWABLE
 where the inward-borders created for each colour area
 are "painted" with a brush newly created from a set
 of chosen patterns
 ............................................................. 
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      
 
 1. save current context
 2. identify the OS where running
 3. if the user did not indicate how many colour areas to use,
    determine the number of areas based on the image size
    but with a minimum of 6 and a maximum of 20
 4. check the existence of at least one pattern satisfying the filtering string   
 5. prepare the new image, copying the source drawable into the new BG
 6. check/adjust mean value to avoid too dark or too light source images
 7. apply a selective gaussian blur to the new BG
 6. duplicate the BG to hold the lines
 7. create a new layer to hold the patterns 
 8. duplicate the out image creating a work image for quantization
 9. create two arrays: one of the areas dimensions, the other one of the mean values
    calculate the averages of the areas dimensions and of the mean values
10. once properly quantized, bring back the work layer into the output image
11. fill the two BG copies with a light gray
12. loop on each colour area
    on areas of not too small dimensions:
    8a. determine brush characteristics on the base ot the area mean value
        and create the brush using a randomly chosen pattern
    8b. call an embedded service routine to create inward paths and stroke them
        (procedure derived from RobA "Mosaic Tile Helper")
        ("curvature" of the stroke kindly provided by TinTran, as suggested by Ofnuts)
    fill areas with too small dimensions with a fixed std pattern ("Small Squares")
13. remove created vectors and created palette
14. stretch colours (if option chosen)
15. create a new layer to detect borders of colour areas
16. fill the borders with a fixed std pattern ("Small Squares")
17. adjust layers positions modes and opacities
18. flatten the new image (if option chosen)
    note: the original source is kept under the created "decorated" layer
    for further processing by the user
    (typical: change "mode" from Normal to Softlight or Multiply or Hardlight
     to get different looks of the result)
19. restore context
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 ---------------------------------------------------------------------------
 other info
 ---------------------------------------------------------------------------
 Required:
 a set of "decorating" patterns  
 (default provided in the package, set of different flowers)
 it's recommended to use patterns with transparent BG (.png)
 
 Acknowledgments:
 RobA: "Mosaic Tile Helper"  
 TinTran+Ofnuts: code for rotating strokes  
 both contributions used inside the internal function BordersInwardRepeatDTP
 
 ---------------------------------------------------------------------------
  
'''
#!/usr/bin/env python
from gimpfu import *
import os
from os.path import expanduser
from os import listdir
import platform
import decimal
import random
import time
import datetime

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############################################################################
# service functions
#---------------------------------------------------------------------------
   
def CreateBrush (Size, Spacing, Pattern, myslash, areameanvalue):
     
    pdb.gimp_context_set_pattern (Pattern)
    (pwidth, pheight, pbpp) = pdb.gimp_pattern_get_info (Pattern)
    tempImage = pdb.gimp_image_new (pwidth, pheight, RGB)
    tempLayer = pdb.gimp_layer_new (tempImage, pwidth, pheight, RGBA_IMAGE,
                                    "temp", 100.0, NORMAL_MODE)
    tempImage.add_layer (tempLayer, 0)
    pdb.gimp_drawable_fill (tempLayer, PATTERN_FILL)
    # try saturate
    pdb.gimp_hue_saturation (tempLayer,0,0,0,100)
    # try taking care of the area mean value
    (meanV,a,b,c,d,e) = pdb.gimp_histogram (tempLayer, HISTOGRAM_VALUE, 0, 255)
    if meanV<160: # dark pattern (remember, there is a lot of transparency)
        if areameanvalue<127:   # check against area mean value
            pdb.gimp_curves_spline (tempLayer, HISTOGRAM_VALUE, 6, (0,0,79,127,255,255)) # try lightening
    else:   # light pattern
        if areameanvalue>127:   # check against area mean value
            pdb.gimp_curves_spline (tempLayer, HISTOGRAM_VALUE, 6, (0,0,127,79,255,255)) # try darkeniong    
    pdb.gimp_image_scale (tempImage, Size, Size)  
    
    #pdb.gimp_display_new (tempImage)
    
    # create brush from pattern image
    d_dir = pdb.gimp_gimprc_query ("gimp_dir")
    BrushName = "DtPB-"+Pattern
    BrushName2 = d_dir+myslash+"brushes"+myslash+BrushName+".gbr"        # changed for compatibility with Linux OS
    pdb.file_gbr_save (tempImage, tempLayer, BrushName2, BrushName, Spacing, BrushName)
    
    pdb.gimp_brushes_refresh()
    
    pdb.gimp_context_set_brush (BrushName)        # DON'T FORGET IT !
    
    pdb.gimp_context_set_paint_method("gimp-pencil")
    pdb.gimp_context_set_dynamics("Track Direction")
    
    return(BrushName)
    
# ======================================================================
# Copied this from ofnuts' ofn-text-along-path.py
def computeOrientedSlope(dx,dy,slope): #This is a better version.
    if abs(slope) > 100000: # very vertical
        return math.atan2(dy,dx) # no perfect, but properly oriented
        # keep dx/dy signs but give then the same ratio as in slope
    return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
# ======================================================================
    
def BordersInwardRepeatDTP (inLayer, inBorderSpacing, inColour, inBrush, inBrushSpacing) :
    
    #(procedure derived from RobA "Mosaic Tile Helper")    
    img = pdb.gimp_item_get_image (inLayer)
    
    width = img.width 
    height = img.height
    
    colourtoggle = 1          
    initial_selection = 0
    
    # work in scratch layer
    scratch_layer = inLayer.copy()
    img.add_layer (scratch_layer, -1)
    # fill the scratch layer with white
    scratch_layer.fill (WHITE_FILL)
    
    while pdb.gimp_selection_is_empty (img) == False:
        if colourtoggle == 0:
            gimp.set_foreground ((255, 255, 255, 255))
            colourtoggle = 1
        else:
            gimp.set_foreground ((0, 0, 0, 255))
            colourtoggle = 0
        pdb.gimp_edit_fill (scratch_layer, FOREGROUND_FILL)     
        pdb.gimp_selection_shrink (img, inBorderSpacing)
        
    # select black
    pdb.gimp_by_color_select (scratch_layer, ((0, 0, 0, 255)), 128, CHANNEL_OP_REPLACE, True, False, 0, False)
    black_channel = pdb.gimp_selection_save (img)
    
    # turn black_channel into a path
    pdb.plug_in_sel2path (img, scratch_layer)
    paths = img.vectors
    newpath = paths[0] 
    # make it visible and active
    newpath.visible = True
    pdb.gimp_image_set_active_vectors (img, newpath)
    # delete scratch layer
    img.remove_layer (scratch_layer)
    gimp.set_foreground (inColour)
    #verify=gimp.get_foreground()
    pdb.gimp_context_set_brush (inBrush)
    pdb.gimp_selection_none (img)
    
    #pdb.gimp_edit_stroke_vectors (inLayer, newpath) #replaced by Tin's code below  <<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    pdb.gimp_context_set_paint_method("gimp-pencil")
    pdb.gimp_context_set_dynamics("Track Direction")
    
    #tin start:--------------------------------------------------------------------------------------------------
    num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(newpath)
    paint_between_length = inBrushSpacing
    for i in range(0,num_strokes):
        stroke_length = pdb.gimp_vectors_stroke_get_length(newpath,stroke_ids[i],0.01)
        dist_at = 1
        while dist_at <= stroke_length:
            x1,y1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(newpath,stroke_ids[i],dist_at-1,0.01)
            x2,y2,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(newpath,stroke_ids[i],dist_at,0.01)
            x3,y3,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(newpath,stroke_ids[i],dist_at+1,0.01)
            points = []
            points.append(x2)
            points.append(y2)
            dist_at += paint_between_length
            #angle = computeThetaWithSlope([x1,y1],[x3,y3],slope) #calculate angle based on slope (v1)
            angle = computeOrientedSlope(x3-x1,y3-y1,slope) #calculate angle based on slope (v2)
            angle = angle/2/math.pi * 360
            if angle > 180:
                angle -= 360
            pdb.gimp_context_set_brush_angle(angle)
            pdb.gimp_paintbrush(inLayer,0,len(points),points,PAINT_CONSTANT,0)
    #tin end. -----------------------------------------------------------------------------------------------------
    
    #pdb.gimp_image_remove_vectors (img, newpath)
 
    #done =======================================================

    return (inLayer)

############################################################################
#   M A I N
############################################################################

def DecorationThruPaths (inImage, inLayer,  
                        widthheight, nrAreas, patFilter,
                        doStretchCol, doFlatten) :
 
    pdb.gimp_context_push()
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    home = expanduser("~")
    # for compatibility with Linux systems
    myplatform=platform.system()
    pdb.gimp_message("I'm running "+myplatform) # please have a look, check if "Apple"   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    if myplatform=="Linux":
        myslash="/"
    if myplatform=="Windows":
        myslash="\\"
    if myplatform=="Apple":
        myslash="/"                             # please have a look, check if correct
    #-----------------------------------------------------------------------------------------------
    # definitions added for 2.9
    # The type of fill
    FOREGROUND=FILL=0; BACKGROUND_FILL=1; WHITE_FILL=2; TRANSPARENT_FILL=3; PATTERN_FILL=4; NO_FILL=5
    # -----------------------------------------------------------------------------------------------
    
    # set sample context
    pdb.gimp_context_set_sample_criterion (SELECT_CRITERION_COMPOSITE)
    pdb.gimp_context_set_sample_merged (False)
    pdb.gimp_context_set_sample_threshold (0.01)
    pdb.gimp_context_set_sample_threshold_int (01)
    pdb.gimp_context_set_sample_transparent (False)
    
    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    name = inLayer.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # delete all brushes generated in a previous run
    Brushes = []
    (CountBrushes, Brushes) = pdb.gimp_brushes_get_list ("DtPB-")
    for bn in range(CountBrushes):
        pdb.gimp_brush_delete (Brushes[bn])
        bn+=1
    pdb.gimp_brushes_refresh()

    # calculate scale factor for the new output image
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight
        if widthheight < 3000:
            widthheight=3000    # minimum
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    # note : original will be scaled back after copy 
    pdb.gimp_image_scale (inImage, rwidth, rheight)
    
    if nrAreas == 0: # auto
        nrAreas = int(optiWidthHeight/400)
        if nrAreas < 6:
            nrAreas = 6
        if nrAreas > 20:
            nrAreas = 20
    looktext = "decorated paths"
    suffix = " on "+str(int(nrAreas))+" areas, look of "+looktext
  
    #   Check patterns set
   
    (numpats, namespats) = pdb.gimp_patterns_get_list(patFilter)
    if numpats==0:
        erro_msg ("no pattern found using your selection string; return.")
        return()
    
# --------------------------------------------------------------------------------------

    newImage = pdb.gimp_image_new (rwidth, rheight, type)                    
    newBG = pdb.gimp_layer_new (newImage, rwidth, rheight, RGBA_IMAGE, "BG", 100.0, NORMAL_MODE)     # background layer
    newImage.add_layer (newBG, 0)
        
    pdb.gimp_edit_copy (inLayer)
    floatsel = pdb.gimp_edit_paste (newBG, True)
    pdb.gimp_floating_sel_anchor (floatsel)
    
    # check/avoid too dark or too light source images
    (meanV,a,b,c,d,e) = pdb.gimp_histogram (newBG, HISTOGRAM_VALUE, 0, 255)
    # prepro steps
    if meanV < 64:      #very dark
        screen1 = newBG.copy()
        screen2 = newBG.copy()
        newImage.add_layer (screen1, 0)
        newImage.add_layer (screen2, 0)
        screen1.mode = SCREEN_MODE
        screen2.mode = SCREEN_MODE
        newBG = newImage.flatten()
    elif meanV < 96:    #dark
        screen1 = newBG.copy()
        newImage.add_layer (screen1, 0)
        screen1.mode = SCREEN_MODE
        newBG = newImage.flatten()
    elif meanV < 128:   #slightly dark
        screen1 = newBG.copy()
        newImage.add_layer (screen1, 0)
        screen1.mode = SCREEN_MODE
        screen1.opacity = 50.0
        newBG = newImage.flatten()
    elif meanV > 192:   #very light
        multiply1 = newBG.copy()
        newImage.add_layer (multiply1, 0)
        multiply1.mode = MULTIPLY_MODE
        newBG = newImage.flatten()
    elif meanV > 160:   #light
        multiply1 = newBG.copy()
        newImage.add_layer (multiply1, 0)
        multiply1.mode = MULTIPLY_MODE
        multiply1.opacity = 50.0
        newBG = newImage.flatten()
    
    #pdb.gimp_display_new (newImage)
    #return
    
    newBG.mode = NORMAL_MODE
    newBG.name = "newBG: Background Image Layer"
    newBG.opacity = 100.0 
    pdb.gimp_layer_flatten (newBG)
    pdb.plug_in_sel_gauss (newImage, newBG, 16, 30)  # preliminary
    
    newBG2 = newBG.copy()
    newImage.add_layer (newBG2, 0)
    newBG2.mode = NORMAL_MODE
    newBG2.name = "newBG2: Image Layer for Lines"
    newBG2.opacity = 100.0
    pdb.gimp_layer_flatten (newBG2) # for stroking?
    
    # gray layer (removed)
    #newBG3 = pdb.gimp_layer_new (newImage, rwidth, rheight, RGBA_IMAGE, "newBG3: Image Layer for Patterns", 100.0, NORMAL_MODE)
    #newImage.add_layer (newBG3, 0)
    
    #pdb.gimp_display_new(newImage)
    #return()
     
    pdb.gimp_image_undo_freeze (newImage)                       # save time and space
    
#####################################################################################################
#   quantize the colour_areas using mode indexed on a duplicated image (WORK)
# =================================================================================================== 

    WORK_IMAGE = pdb.gimp_image_duplicate (newImage)
    WORK_DRAWABLE = WORK_IMAGE.layers[1]
    WORK_DRAWABLE.name = "BG Layer"
    
    # change image mode to indexed (this produces a cartoonized image, nrAreas colors)
    
    pdb.gimp_brightness_contrast (WORK_DRAWABLE, +0, +15)   # indexed mode creates "opaque" colours
    pdb.gimp_image_convert_indexed (WORK_IMAGE, 0, 0, nrAreas, 0, 0, "ignored")    # INPUT   <<<<<<<< 
    is_indexed = pdb.gimp_drawable_is_indexed (WORK_DRAWABLE) 
    if is_indexed == 1:
        hidden_message = "OK, converted"
    else:    
        erro_msg ("ERROR! image not converted, exit")
        return
    #   
    # get the number of pixels in the colormap (which exists for indexed images)
    # ==========================================================================
    (count, colors) = pdb.gimp_image_get_colormap (WORK_IMAGE)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas = (count/3) 
    #info_msg ("nr of colors in the palette: "+str(nr_areas)) 
  
    areas_palette = pdb.gimp_palette_new(pdb.gimp_image_get_name (WORK_IMAGE))
    
    index = 0
    while index < count:
    # Compute the color
        color = (colors[index], colors[index+1], colors[index+2])
    # Add to the filter temporary palette
        pdb.gimp_palette_add_entry (areas_palette, str(index/3), color)
        index+=3
        
    # Endwhile index<count
    # the temporary palette has been created            
    # ======================================

    # re-set image to RGB to proceed
    # ==============================
    pdb.gimp_image_convert_rgb (WORK_IMAGE)
    pdb.gimp_layer_add_alpha (WORK_DRAWABLE)                        #  RGBA
    pdb.gimp_selection_none (WORK_IMAGE)
    #pdb.gimp_display_new (WORK_IMAGE)
    #return

    # =========================================
    # loop to get the luminosity of the N areas                 
    # =========================================

    areaNR = 0
    sizes = []
    lumas = []
    sumsizes = 0
    sumlumas = 0
    for areaNR in range (0, nr_areas):
        one_color = pdb.gimp_palette_entry_get_color (areas_palette, areaNR)
        pdb.gimp_image_select_color (WORK_IMAGE, CHANNEL_OP_REPLACE, WORK_DRAWABLE, one_color)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds (WORK_IMAGE)
        if there_is_a_selection == 1:
            (mean, stddev, median, pixels, count, percent) = pdb.gimp_histogram (WORK_DRAWABLE, 0, 0, 255)
            sizes.append (int(round(pixels)))
            sumsizes += int(round(pixels))
            lumas.append (int(round(mean)))
            sumlumas += int(round(mean))
        meansize = int(sumsizes/nr_areas)
        meanluma = int(sumlumas/nr_areas)
    #info_msg ("end loop")
    #info_msg ("sizes = "+str(sizes))
    #info_msg ("lumas = "+str(lumas))
    
    #pdb.gimp_display_new (WORK_IMAGE)
    #return

    # ========================  
    # loop to DRAW the N areas  
    # for BordersInwardRepeat
    # ======================== 

    newWORK = pdb.gimp_layer_new_from_drawable (WORK_DRAWABLE, newImage)
    newImage.add_layer (newWORK, 0)
    newWORK.name = "newWORK: Working Layer to hold Colour Areas"
    newWORK.opacity = 100.0
    newWORK.mode = GRAIN_MERGE_MODE
    
    #pdb.gimp_display_new (newImage)
    #return
    
    papercolour = (167, 166, 165, 255)
    gimp.set_foreground (papercolour)
    pdb.gimp_edit_fill (newBG2, FOREGROUND_FILL)
    #pdb.gimp_edit_fill (newBG3, FOREGROUND_FILL)  # try removing gray layer

    #pdb.gimp_display_new (newImage)
    #return
    areaNR = 0
    area_color = []   # empty list
    pencolour = (0,0,0,255) # default
    for areaNR in range (0, nr_areas):
    #for areaNR in range (0, 2):
        one_color = pdb.gimp_palette_entry_get_color (areas_palette, areaNR)
        area_color.append (one_color)
        (Ra,Ga,Ba,Aa) = one_color
        pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, newWORK, one_color)
        pdb.gimp_selection_shrink (newImage, 2)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds (newImage)
        if (there_is_a_selection == 1):    
              
            pdb.gimp_selection_shrink (newImage, int(widthheight/nrAreas/20))   # 1/4 brushsize, see below
            (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds (newImage)

            if (there_is_a_selection == 1):
                (mean, stddev, median, pixels, count, percent) = pdb.gimp_histogram (newWORK, 0, 0, 255)
                if pixels > (meansize/100): # do not apply to very small areas
                    
                    #brushsize = 100 # try fixed
                    brushsize = int(widthheight/nrAreas/5)   # try relative
                    #brushspacing  = 50 # try fixed %
                    brushspacing  = 60 # try fixed %
                    #info_msg ("brushsize = "+str(brushsize)+" brushspacing = "+str(brushspacing))
                    MeanValue = (Ra+Ga+Ba)/3
                    if MeanValue<127:
                        RaI=max(0,Ra-64); GaI=max(0,Ga-64);BaI=max(0,Ba-64) # try always darker
                    else:
                        RaI=max(0,Ra-96); GaI=max(0,Ga-96);BaI=max(0,Ba-96) # try always darker
                    RaI=max(0,Ra-16); GaI=max(0,Ga-16);BaI=max(0,Ba-16) # try
                    
                    onepat = random.randint (0, numpats-1)
                    thisbrush = CreateBrush (brushsize, brushspacing, namespats[onepat], myslash, MeanValue)
                    
                    borderspacing = brushsize*0.5  # relative
                    lumavalue = lumas[areaNR]
                    if lumavalue>240:
                        lumavalue = 240
                    if lumavalue<20:
                        lumavalue = 20
                    pencolour = (RaI,GaI,BaI,255)
                    #info_msg ("pencolour = "+str(RaI)+","+str(GaI)+","+str(BaI)+";")
                    brushdistance = brushsize*brushspacing/100
                    pdb.gimp_context_set_brush_size (brushsize) # DON'T FORGET IT !
                    BordersInwardRepeatDTP (newBG2, borderspacing, pencolour, thisbrush, brushdistance)
                      
                else:
                    info_msg ("AREA SKIPPED: pixels = "+str(int(pixels))+
                              ", with a limit of "+str(int(meansize/16)))
                    pdb.gimp_context_set_pattern ("Small Squares")
                    pdb.gimp_edit_fill (newBG2, PATTERN_FILL)
            
    #info_msg ("end loop")
    
    #newBG3.mode = GRAIN_MERGE_MODE    # try removing gray layer
    
    #pdb.gimp_display_new (newImage)
    #return
    
    Vectors = newImage.vectors
    CountVectors = len(Vectors)
    for vn in range(CountVectors):
        thisVector = Vectors[vn] 
        # make it visible and active
        thisVector.visible = True
        pdb.gimp_image_set_active_vectors (newImage, thisVector)
        pdb.gimp_image_remove_vectors (newImage, thisVector)
        
    pdb.gimp_palette_delete(areas_palette)
    pdb.gimp_selection_none (newImage)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    if doStretchCol == True:
        pdb.gimp_curves_spline (newWORK, HISTOGRAM_VALUE, 34,        
                               ( 0, 15,  15, 0,  31, 47,  47, 31,
                                63, 79,  79, 63,  95, 111, 111, 95,
                               127, 143, 143, 127, 159, 175, 175, 159, 
                               191, 207, 207, 191, 223, 239, 239, 223,
                               255, 255)
                               )
         
    newCONTOURS = newWORK.copy()
    newImage.add_layer (newCONTOURS, 0)
    newCONTOURS.mode = MULTIPLY_MODE
    newCONTOURS.name = "newCONTOURS: Image Layer for Contours"
    newCONTOURS.opacity = 100.0
    # std edge detect  
    pdb.plug_in_edge (newImage, newCONTOURS, 10.0, 2, 0)
    pdb.gimp_desaturate_full (newCONTOURS, DESATURATE_LUMINOSITY)
    pdb.plug_in_vinvert (newImage, newCONTOURS)
    pdb.gimp_threshold (newCONTOURS, 127, 255)
    
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, newCONTOURS, ((255,255,255,255)))
    pdb.gimp_edit_cut (newCONTOURS)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, newCONTOURS, ((0,0,0,255)))
    pdb.gimp_selection_grow (newImage, int(widthheight/1000))
    gimp.set_foreground (pencolour)
    pdb.gimp_edit_fill (newCONTOURS, FOREGROUND_FILL)   
    #onepat = random.randint (0, numpats-1)
    #pdb.gimp_context_set_pattern (namespats[onepat])
    pdb.gimp_context_set_pattern ("Small Squares")
    pdb.gimp_edit_fill (newCONTOURS, PATTERN_FILL)
    pdb.gimp_brightness_contrast (newCONTOURS, -50, +50)
    pdb.plug_in_spread (newImage, newCONTOURS, 0.7, 0.7)
    pdb.gimp_selection_none (newImage)
    # remove gray from BG2
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, newBG2, ((papercolour)))
    pdb.gimp_layer_add_alpha (newBG2)
    ####pdb.gimp_edit_cut (newBG2)
    pdb.gimp_selection_none (newImage)
    pdb.gimp_item_set_visible (newBG, False)
    # source on top
    pdb.gimp_image_raise_item_to_top (newImage, newWORK)
    pdb.gimp_hue_saturation (newWORK, 0, 0, +25, +45)
    newWORK.mode = SOFTLIGHT_MODE    
    newWORK.opacity = 50.0
    # dup source
    newWORK2 = newWORK.copy()
    newImage.add_layer (newWORK2, 0)
    newWORK2.name = "newWORK: 2nd WORK Layer"
    newWORK2.mode = HARDLIGHT_MODE   
    newWORK2.opacity = 95.0
    # invert values
    pdb.plug_in_vinvert (newImage, newWORK)
    
    # visible TOP
    newTOP = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible on Top")
    newImage.add_layer (newTOP, 0)
    pdb.gimp_item_set_visible (newBG, True)
    newTOP.mode = SOFTLIGHT_MODE
    pdb.plug_in_vinvert (newImage, newTOP)
    # using a low opacity
    newTOP.opacity = 25.0
    
    #pdb.gimp_display_new(newImage)
    #return()
    #
#================================================================================================ 
    #  
    topBG = newBG.copy()
    newImage.add_layer (topBG, 1)
    topBG.mode = NORMAL_MODE
    topBG.name = "topBG: Image BG Layer for merging"
    topBG.opacity = 50.0
    pdb.gimp_item_set_visible (newBG, False)
    
    if doFlatten == True:
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
    else:
        Outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible Outcome on Top")
        newImage.add_layer (Outcome, 0)
        
    pdb.gimp_item_set_visible (newBG, True)
    
    Outcome.mode = NORMAL_MODE
    Outcome.opacity = 100.0
    
    # ----------------------------------------------------------------------------------------------- 
    Outcome.name = st+" "+name+suffix
    #pdb.gimp_display_new(newImage)
    #return()
    
    pdb.gimp_image_scale (inImage, owidth, oheight)                 # source back to original size
    pdb.gimp_display_new(newImage)
    pdb.gimp_displays_flush()
    #return()

    pdb.gimp_context_pop()
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    pdb.gimp_image_delete (WORK_IMAGE)
    
    #return()
    return (Outcome)

# =====================================================================================================

register(
    "DecorationThruPaths",    
    "to create an image having decorations using paths",   
    "This script creates an image having decorations using paths.",
    "Diego", 
    "Diego Nassetti ", 
    "November 2017",
    "DecorationThruPaths", 
    "RGB*,GRAY*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)"+
                              "\n (3000 minimum)", 4000, (0, 8000, 200)),
      (PF_ADJUSTMENT, "colorareas", "How many colour areas"+"\n (0=auto)"+
                              "\n (6-20 recommended)", 0, (0, 64, 1)),
      (PF_STRING,"patterns",   "Patterns selection string:", "FlowerPng"), 
      (PF_TOGGLE, "stretchcolor", "Stretch Colours?", False),
      (PF_TOGGLE, "flatten", "Flatten Outcome?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    DecorationThruPaths,
    menu="<Image>/Diego/Artistic",
    )

main()